package de.uni_leipzig.asv.unsupos;

import java.util.Locale;

/**
 * This class implements some time measuring facilities.
 * 
 * @author  Andreas Klaus
 * @version 0.1
 */
public class Timer {
	private Locale loc = new Locale("en", "US");
	
	/**
	 * These variables are used for timing. 
	 */
    private long begin, end;
    
    /**
     * Default constructor: starts the time measurement.
     */
    public Timer() {
    	begin = System.currentTimeMillis(); }

    /**
     * Start time measurement.
     */
    public void start() {
    	begin = System.currentTimeMillis(); }
    
    /**
     * Returns the measured time difference as a String.
     * 
     * @return the measured time difference as a String.
     */
    public String toString() {
    	
    	/* The time difference. */
    	double fsecs = 0.001 * (end - begin);
    	int    tdiff;
    	
    	/* The string to be returned. */
    	String time = "-- seconds";
    	
    	if (fsecs >= 0.0) {
      	
    	/* Seconds. */
      	if (fsecs < 60.0) {
    	  	time = String.format(Locale.US, "%1$.3f seconds", fsecs); }
      	
      	/* Minutes. */
    	else if ((fsecs / 60.0) < 60.0) {
    		tdiff = (int) fsecs; /* in seconds */
          	time = (tdiff / 60) + " minute(s), " + (tdiff % 60) + " second(s)"; }
    	
      	/* Hours. */
      	else if ((fsecs / 3600.0) < 24.0) {
      		tdiff = (int) (fsecs / 60.0); /* in minutes */
      		time = (tdiff / 60) + " hour(s), " + (tdiff % 60) + " minute(s)"; }
         	
        /* Days. */
        else {
        	tdiff = (int) (fsecs / 3600.0);
        	time = (tdiff / 24) + " day(s), " + (tdiff % 24) + " hour(s)"; } }
      
    	return time; }
    
    /**
     * Stop time measurement.
     */
    public void stop() {
    	end = System.currentTimeMillis(); }
    
    /**
	   * Prints the measured time in seconds.
	   */
  	public void printTimeSec() {
      Object[] x = new Object[1];
  	  x[0] = new Double(0.001 * (end - begin));
  	  System.out.printf(loc, "%.3f Seconds\n", x); }
}
