/**
 * This file was adapted for the UnsuPos system. Some annotations <code>@SuppressWarnings("unchecked")</code>
 * were added. Furthermore the <code>printDBResults()</code> method of the <code>RankedResultList</code>
 * class was modified to write only necessary information.
 * 
 * @author Andreas Klaus 
 */

package de.uni_leipzig.asv.unsupos.similarity;

import java.util.*;

/**
 * Merges two or more vectors into one in order to be able to compare items by
 * one vector.
 *
 * @author Stefan Bordag
 */
public class Merger
{
  private Merger()
  {
  }

  /**
   * Merges all the given maps into the last one, assuming that there is more
   * then one.
   * @param maps Map[]
   * @return Map
   */
  public static Map mergeToLastMap(Map[] maps)
  {
    if ( maps == null || maps.length < 1 )
    {
      return new HashMap();
    }
    if ( maps.length == 1 )
    {
      return maps[0];
    }
    for ( int i = 0 ; i < maps.length -1 ; i++ )
    {
      maps[maps.length-1] = mergeToMap2(maps[i],maps[maps.length-1]);
    }
    return maps[maps.length-1];
  }

  /**
   * A map is defined to have Integers as keys and Integer[] as values.
   * The merge occurs for each Integer[i].
   * Warning: The map2 will be used as the return container (and thus be
   * modified) in order to prevent unnecessary creation of objects
   *
   * @param map1 Map the one map, will remain unmodified
   * @param map2 Map the other map, will be modified!
   * @return Map
   */
  @SuppressWarnings("unchecked")
  public static Map mergeToMap2(Map map1, Map map2)
  {
    // Going through the first map in order to ensure that any item will be in
    // the result because the second map wil anyway be returned.
    for ( Iterator it = map1.keySet().iterator() ; it.hasNext() ; )
    {
      Integer curKey = (Integer)it.next();
      if ( map2.containsKey(curKey) )
      {
        Integer[] values1 = (Integer[])map1.get(curKey);
        Integer[] values2 = (Integer[])map2.get(curKey);
        values2 = mergeValues(values1,values2);
        map2.put(curKey, values2);
      }
      else
      {
        map2.put(curKey,map1.get(curKey));
      }
    }
    return map2;
  }

  /**
   * Merges the values into one. The value arrays are assumed to be equally
   * long.
   *
   * @param vals1 Integer[] the one values
   * @param vals2 Integer[] the other values - will be modified
   * @return Integer[]
   */
  public static Integer[] mergeValues(Integer[] vals1, Integer[] vals2)
  {
    if ( vals1.length != vals2.length )
    {
      throw new IllegalArgumentException("In Merger.mergeValues two value fields occured which had various lengthes. Please check input files.");
    }
    for ( int i = 0 ; i < vals1.length ; i ++ )
    {
      vals2[i] = new Integer( vals1[i].intValue() + vals2[i].intValue() ) ;
    }
    return vals2;
  }

}
