/**
 * This file was adapted for the UnsuPos system. Some annotations <code>@SuppressWarnings("unchecked")</code>
 * were added. Furthermore the <code>printDBResults()</code> method of the <code>RankedResultList</code>
 * class was modified to write only necessary information.
 * 
 * @author Andreas Klaus 
 */

package de.uni_leipzig.asv.unsupos.similarity;

//import de.uni_leipzig.asv.wordserver.*;

import java.util.*;
import java.io.*;

/**
 * Whenever KollokSimilarity calculates one more result for a word, it puts it
 * here. When done completely with a given word, it calls print and then
 * destroys this object.
 *
 * @author Stefan Bordag
 * @date   19.11.2003
 */
public class RankedResultList
{
  protected Set lists = null;

  protected int maxSize = 400;

  public static boolean DIR_BASELINE = false; // desc
  public static boolean DIR_BASELINE_NORM = false; // desc
  public static boolean DIR_DICE = false; // desc
  public static boolean DIR_JACCARD = false; // desc
  public static boolean DIR_COS = false; // desc
  public static boolean DIR_CITY = true; // asc
  public static boolean DIR_EUKLID = true; // asc
  public static boolean DIR_COS_BIN = false; // desc
  public static boolean DIR_JS = true; // asc

  
  private SimOptions simopt = null;
  
//  protected WordServer wserver = null;

  @SuppressWarnings("unchecked")
  public RankedResultList(SimOptions simopt)//WordServer ws)
  {
	this.simopt = simopt;
    this.maxSize = this.simopt.getMaxResultsSizePerWord();
    this.lists = new HashSet();

    int[] ordering = {SimOptions.BASELINE,SimOptions.BASELINE_NORM,SimOptions.DICE, SimOptions.JACCARD, SimOptions.COS, SimOptions.CITY, SimOptions.EUKLID, SimOptions.COS_BIN,SimOptions.JENNSEN_SHANNON};
    boolean[] direction = {DIR_BASELINE,DIR_BASELINE_NORM,DIR_DICE,DIR_JACCARD,DIR_COS,DIR_CITY,DIR_EUKLID,DIR_COS_BIN,DIR_JS};
    TreeSet list = new TreeSet(new ResultSorter(ordering, direction));
    this.lists.add(list);
    int[] ordering2 = {SimOptions.BASELINE_NORM,SimOptions.BASELINE,SimOptions.DICE, SimOptions.JACCARD, SimOptions.COS, SimOptions.CITY, SimOptions.EUKLID, SimOptions.COS_BIN,SimOptions.JENNSEN_SHANNON};
    boolean[] direction2 = {DIR_BASELINE_NORM,DIR_BASELINE,DIR_DICE,DIR_JACCARD,DIR_COS,DIR_CITY,DIR_EUKLID,DIR_COS_BIN,DIR_JS};
    list = new TreeSet(new ResultSorter(ordering2, direction2));
    this.lists.add(list);
    int[] ordering3 = {SimOptions.DICE, SimOptions.JACCARD, SimOptions.BASELINE,SimOptions.BASELINE_NORM,SimOptions.COS, SimOptions.CITY, SimOptions.EUKLID, SimOptions.COS_BIN,SimOptions.JENNSEN_SHANNON};
    boolean[] direction3 = {DIR_DICE,DIR_JACCARD,DIR_BASELINE,DIR_BASELINE_NORM,DIR_COS,DIR_CITY,DIR_EUKLID,DIR_COS_BIN,DIR_JS};
    list = new TreeSet(new ResultSorter(ordering3, direction3));
    this.lists.add(list);
    int[] ordering4 = {SimOptions.JACCARD, SimOptions.DICE,SimOptions.BASELINE,SimOptions.BASELINE_NORM, SimOptions.COS, SimOptions.CITY, SimOptions.EUKLID, SimOptions.COS_BIN,SimOptions.JENNSEN_SHANNON};
    boolean[] direction4 = {DIR_JACCARD,DIR_DICE,DIR_BASELINE,DIR_BASELINE_NORM,DIR_COS,DIR_CITY,DIR_EUKLID,DIR_COS_BIN,DIR_JS};
    list = new TreeSet(new ResultSorter(ordering4, direction4));
    this.lists.add(list);
    int[] ordering5 = {SimOptions.COS, SimOptions.COS_BIN,  SimOptions.CITY, SimOptions.EUKLID,SimOptions.BASELINE,SimOptions.BASELINE_NORM,SimOptions.DICE, SimOptions.JACCARD,SimOptions.JENNSEN_SHANNON};
    boolean[] direction5 = {DIR_COS,DIR_COS_BIN,DIR_CITY,DIR_EUKLID,DIR_BASELINE,DIR_BASELINE_NORM,DIR_DICE,DIR_JACCARD,DIR_JS};
    list = new TreeSet(new ResultSorter(ordering5, direction5));
    this.lists.add(list);
    int[] ordering6 = {SimOptions.CITY, SimOptions.EUKLID, SimOptions.COS, SimOptions.BASELINE,SimOptions.BASELINE_NORM,SimOptions.DICE, SimOptions.JACCARD, SimOptions.COS_BIN,SimOptions.JENNSEN_SHANNON};
    boolean[] direction6 = {DIR_CITY,DIR_EUKLID,DIR_COS,DIR_BASELINE,DIR_BASELINE_NORM,DIR_DICE,DIR_JACCARD,DIR_COS_BIN,DIR_JS};
    list = new TreeSet(new ResultSorter(ordering6, direction6));
    this.lists.add(list);
    int[] ordering7 = {SimOptions.EUKLID, SimOptions.CITY, SimOptions.COS, SimOptions.BASELINE,SimOptions.BASELINE_NORM,SimOptions.DICE, SimOptions.JACCARD, SimOptions.COS_BIN,SimOptions.JENNSEN_SHANNON};
    boolean[] direction7 = {DIR_EUKLID,DIR_CITY,DIR_COS,DIR_BASELINE,DIR_BASELINE_NORM,DIR_DICE,DIR_JACCARD,DIR_COS_BIN,DIR_JS};
    list = new TreeSet(new ResultSorter(ordering7, direction7));
    this.lists.add(list);
    int[] ordering8 = {SimOptions.COS_BIN, SimOptions.COS, SimOptions.EUKLID, SimOptions.CITY, SimOptions.BASELINE,SimOptions.BASELINE_NORM,SimOptions.DICE, SimOptions.JACCARD,SimOptions.JENNSEN_SHANNON};
    boolean[] direction8 = {DIR_COS_BIN,DIR_COS,DIR_EUKLID,DIR_CITY,DIR_BASELINE,DIR_BASELINE_NORM,DIR_DICE,DIR_JACCARD,DIR_JS};
    list = new TreeSet(new ResultSorter(ordering8, direction8));
    this.lists.add(list);

    int[] ordering9 = {SimOptions.JENNSEN_SHANNON, SimOptions.COS_BIN, SimOptions.COS, SimOptions.EUKLID, SimOptions.CITY, SimOptions.BASELINE,SimOptions.BASELINE_NORM,SimOptions.DICE, SimOptions.JACCARD};
    boolean[] direction9 = {DIR_JS, DIR_COS_BIN,DIR_COS,DIR_EUKLID,DIR_CITY,DIR_BASELINE,DIR_BASELINE_NORM,DIR_DICE,DIR_JACCARD};
    list = new TreeSet(new ResultSorter(ordering9, direction9));
    this.lists.add(list);

//    this.wserver = ws;
    try
    {
      Integer maxSize = new Integer(this.simopt.getMaxResultsSizePerWord());
      this.maxSize = maxSize.intValue();
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
    }
  }

  /**
   * Seeks the correct place in the list, puts the element there and removes the
   * last element.
   * Words in this list are ordered by anz first, then cos, then anz_norm.
   *
   * @Todo: Do some better implementation, like binary search or sth.
   *
   * @param wordNr1
   * @param wordNr2
   * @param anz
   * @param anz_norm
   * @param cos
   */
  @SuppressWarnings("unchecked")
  public void putNextResult(Integer wordNr1, Integer wordNr2, double[] sim)
  {
    if ( wordNr1.intValue() == wordNr2.intValue() )
    {
      return;
    }
    //System.out.println("Putting next result: "+wordNr1+" "+wordNr2+" "+sim[SimOptions.BASELINE]);
    ResultElement el = new ResultElement(wordNr1, wordNr2, sim);
//    int i = 0;
    for ( Iterator it = this.lists.iterator() ; it.hasNext() ; /*i++*/ )
    {
      TreeSet curList = (TreeSet)it.next();
/*      if ( i == 0 )
      {
        System.out.println("Anzahllist: "+curList);
      }*/
      curList.add(el);
      while ( curList.size() > this.maxSize )
      {
        curList.remove(curList.last());
      }
    }
  }

  /**
   * Merges lists into one (to avoid double entries) and prints that one
   */
  @SuppressWarnings("unchecked")
  public void printResults()
  {
    try
    {
      String fileName = simopt.getOutputMainString();
      BufferedWriter writer = new BufferedWriter(new FileWriter(fileName, true));

      Set set1 = new HashSet(this.maxSize*2);
      Set set2 = new HashSet(this.maxSize*2);

      for ( Iterator it = this.lists.iterator() ; it.hasNext() ; )
      {
        TreeSet curList = (TreeSet)it.next();
        for ( Iterator it2 = curList.iterator() ; it2.hasNext() ; )
        {
          ResultElement curEl = (ResultElement)it2.next();
          // for each input word only one unique pair of result words should occur
          if ( set1.contains(curEl.wordNr1) && set2.contains(curEl.wordNr2) )
          {
            // this is a double, ignore (or count it for overlapping statistics.)
            // (should overlapt a lot)
          }
          else
          {
            printDBResults(curEl,writer);
            //writer.write(printDBResults(curEl));
            set1.add(curEl.wordNr1);
            set2.add(curEl.wordNr2);
          }
        }
      }
      writer.close();
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
    }
  }

  public void printDBResults(ResultElement curEl, BufferedWriter writer) throws IOException
  {
		writer.write(curEl.wordNr1            + "\t" +
				     curEl.wordNr2            + "\t" +
				     Math.round(curEl.sim[0]) + "\n");
	    writer.write(curEl.wordNr2            + "\t" +
	    		     curEl.wordNr1            + "\t" +
	    		     Math.round(curEl.sim[0]) + "\n");
    /*
	writer.write(curEl.wordNr1 + "\t" + curEl.wordNr2 + "\t");
    for ( int i = 0 ; i < curEl.sim.length ; i++ )
    {
      writer.write(""+Math.round(curEl.sim[i]));
      if ( i < curEl.sim.length -1 )
      {
        writer.write("\t");
      }
      else
      {
        writer.write("\n");
      }
    }
    writer.write(curEl.wordNr2 + "\t" + curEl.wordNr1 + "\t");
    for ( int i = 0 ; i < curEl.sim.length ; i++ )
    {
      writer.write(""+Math.round(curEl.sim[i]));
      if ( i < curEl.sim.length -1 )
      {
        writer.write("\t");
      }
      else
      {
        writer.write("\n");
      }
    } */
  }

  public String printDBResultsString(ResultElement curEl)
  {
    String s1 = curEl.wordNr1 + "\t" + curEl.wordNr2 + "\t";
    String s2 = curEl.wordNr2 + "\t" + curEl.wordNr1 + "\t";
    for ( int i = 0 ; i < curEl.sim.length ; i++ )
    {
      s1 = s1 + Math.round(curEl.sim[i]);
      s2 = s2 + Math.round(curEl.sim[i]);
      if ( i < curEl.sim.length -1 )
      {
        s1 = s1 + "\t";
        s2 = s2 + "\t";
      }
      else
      {
        s1 = s1 + "\n";
        s2 = s2 + "\n";
      }
    }
    return s1+s2;
  }

/*  public void printWords(String fileName)
  {
    int factor = SimOptions.getInstance().getResultsFactor();
    try
    {
      FileWriter writer = new FileWriter( fileName + "_words.txt", true );
      for ( Iterator it = this.list.iterator(); it.hasNext(); )
      {
        ResultElement curEl = ( ResultElement )it.next();

        writer.write( this.wserver.getWordForNumber( curEl.wordNr1 ) +
                            "\t" + this.wserver.getWordForNumber( curEl.wordNr2 ) +
                            "\t" + ( int )curEl.sim[0] + "\t" +
                            ( ( int ) ( curEl.sim[1] * factor ) ) + "\t" +
                            ( ( int ) ( curEl.sim[2] * factor ) ) + "\t" +
                            ( ( int ) ( curEl.sim[3] ) ) + "\t" +
                            ( ( int ) ( curEl.sim[4] * factor ) ) +"\n" );
      }
      writer.close();
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
    }
  }*/

  class ResultSorter implements Comparator
  {
    public int[] columns = null;
    public boolean[] orderings = null;

    public ResultSorter(int[] myColumns, boolean[] orderings)
    {
      this.columns = myColumns;
      this.orderings = orderings;
      if ( this.columns.length < this.orderings.length )
      {
        throw new IllegalArgumentException("In RankedResultsList.ResultSorter: Constructed ResultSorter with a different sized columns then orderings.");
      }
    }

    public int compare(Object obj1, Object obj2)
    {
      int DIR1 = -1;
      int DIR2 = 1;

      if ( obj1 instanceof ResultElement && obj2 instanceof ResultElement )
      {
        ResultElement res = (ResultElement)obj1;
        ResultElement res2 = (ResultElement)obj2;

        for ( int i = 0 ; i < this.columns.length ; i++ )
        {
          if ( res.sim[this.columns[i]] > res2.sim[this.columns[i]] )
          {
            if ( !this.orderings[i] )
            {
              return DIR1;
            }
            else
            {
              return DIR2;
            }
          }
          else if ( res.sim[this.columns[i]] < res2.sim[this.columns[i]] )
          {
            if ( !this.orderings[i] )
            {
              return DIR2;
            }
            else
            {
              return DIR1;
            }
          }
        }
      }
      return 0;
    }

    public boolean equals(Object obj)
    {
      return this.equals(obj);
    }

    public String toString()
    {
      String retString = "(";
      for ( int i = 0 ; i < this.columns.length ; i++ )
      {
        retString = retString + this.columns[i];
        if ( i < this.columns.length -1 )
        {
          retString = retString + ".";
        }
        else
        {
          retString = retString + ")";
        }
      }
      return retString;
    }
  }


  /**
   * To make a convenient ordering of the result element, these are formalized
   * into a class which implements the Comparable interface.
   *
   * @author Stefan Bordag
   * @date   19.11.2003
   */
  class ResultElement
  {
    protected Integer wordNr1 = null;
    protected Integer wordNr2 = null;
    protected double sim[] = null;

    public ResultElement(Integer wordNr1, Integer wordNr2, double[] sim)
    {
      this.wordNr1 = wordNr1;
      this.wordNr2 = wordNr2;
      this.sim = sim;
    }

    public String toString()
    {
      return "("+wordNr1+","+wordNr2+","+sim[0]+")\n";//+","+sim[1]+","+sim[2]+","+sim[3]+","+sim[4]+")";
    }
  }

}
