/**
 * This file was adapted for the UnsuPos system. Some annotations <code>@SuppressWarnings("unchecked")</code>
 * were added. Furthermore the <code>printDBResults()</code> method of the <code>RankedResultList</code>
 * class was modified to write only necessary information.
 * 
 * @author Andreas Klaus 
 */

package de.uni_leipzig.asv.unsupos.similarity;

/**
 * Options to make optionable:
 *
 * @author Stefan Bordag
 * @date   19.11.2003
 */
public class SimOptions //extends Options
{
  public boolean debug = false;

  private boolean loadCueToRam = false; // whether to load the cue file into the ram
  private boolean loadFeaturesToRam = false; // whether to load the feature files into the ram
  private boolean showProgress = false; // whether to show progress

  private int minSignificance = 1;     // The minimum significance value for accepted features
  private int minWordNr = 0;           // Ignore all words with word number lower than this (only meaningful for ranknumbers)
  private int maxResultSize = 10000;    // Store at max that many results for each word
  private int resultsFactor = 100000;  // the factor small realvalued numbers are multiplied with
  private int inputDataFactor = 1;   // by what the input data values are multiplied (and this need to be divided again)
  private int minAnzahl = 0;           // how many features two vectors have to have in common to be included in the result list

  private String outputMainString = "results";


  public int maxIndexNr = 100000;      // what's the largest possible word number
  public int minIndexNr = 0;           // what's the smallest possible word number
  public boolean takeFurther = true;   // whether to looks at neighbours of neighbours for candidates
  public int maxCooccsCandidate = 50; // how many neighbours to take for candidate search
  public int maxCooccsFeatures = 100;  // how many features to take for comparison

  public int[] minCols = null;  // for each column a specification which values min
  public int[] maxCols = null;  // for each column a specification which values max

  public static final int BASELINE = 0;
  public static final int BASELINE_NORM = 1;
  public static final int DICE = 2;
  public static final int JACCARD = 3;
  public static final int COS = 4;
  public static final int CITY = 5;
  public static final int EUKLID = 6;
  public static final int COS_BIN = 7;
  public static final int JENNSEN_SHANNON = 8;

  public static final int MEASURES = 9;

  public final static int cols = 3; // how many columns to select from source data.

  public String toString()
  {
    String retString = "Options are now set to:\n";
    retString += "loadCueToRam       = "+this.loadCueToRam+"\n";
    retString += "loadFeaturesToRam  = "+this.loadFeaturesToRam+"\n";
    retString += "minSignificance    = "+this.minSignificance+"\n";
    retString += "minWordNr          = "+this.minWordNr+"\n";
    retString += "maxResultSize      = "+this.maxResultSize+"\n";
    retString += "resultsFactor      = "+this.resultsFactor+"\n";
    retString += "inputDataFactor    = "+this.inputDataFactor+"\n";
    retString += "minAnzahl          = "+this.minAnzahl+"\n";
    retString += "outputMainString   = "+this.outputMainString+"\n";
    retString += "maxIndexNr         = "+this.maxIndexNr+"\n";
    retString += "minIndexNr         = "+this.minIndexNr+"\n";
    retString += "takeFurther        = "+this.takeFurther+"\n";
    retString += "maxCooccsCandidate = "+this.maxCooccsCandidate+"\n";
    retString += "maxCooccsFeatures  = "+this.maxCooccsFeatures+"\n";
    retString += "Output format is: wordNr1, wordNr2, Baseline, base_norm, dice, jaccard, cos, city, euclid, cos_bin, JS\n";
    return retString;
  }

  public SimOptions(){}

  /**
   * The sole instance of options for the whole program
   */
  private SimOptions instance = null;

  public SimOptions getInstance()
  {
    if ( instance == null )
    {
      synchronized(SimOptions.class)
      {
        instance = new SimOptions();
      }
    }
    return instance;
  }


  public int getInputDataFactor()
  {
    return this.inputDataFactor;
  }
  public void setInputDataFactor(int newVal)
  {
    this.inputDataFactor = newVal;
  }
  public int getFactor()
  {
    return this.resultsFactor;
  }
  public void setFactor(int newVal)
  {
    this.resultsFactor = newVal;
  }

  public boolean getTakeFurther()
  {
    return this.takeFurther;
  }
  public void setTakeFurther(boolean take)
  {
    this.takeFurther = take;
  }

  public int getMinSignifikance()
  {
    return this.minSignificance;
  }
  public void setMinSignifikance(int minSig)
  {
    this.minSignificance = minSig;
  }

  public boolean getLoadCueToRam()
  {
    return this.loadCueToRam;
  }
  public void setLoadCueToRam(boolean load)
  {
    this.loadCueToRam = load;
  }

  public boolean getLoadFeaturesToRam()
  {
    return this.loadFeaturesToRam;
  }
  public void setLoadFeaturesToRam(boolean load)
  {
    this.loadFeaturesToRam = load;
  }


  public boolean getShowProgress()
  {
    return this.showProgress;
  }
  public void setShowProgress(boolean show)
  {
    this.showProgress = show;
  }


  /**
   * How many cooccurrences as direct neighbours to use when looking for candidates
   * @return int
   */
  public int getMaxCooccsCandidate()
  {
    return this.maxCooccsCandidate;
  }
  public void setMaxCooccsCandidate(int newVal)
  {
    this.maxCooccsCandidate = newVal;
  }

  /**
   * How many features to use for comparisons.
   * @param newVal int
   */
  public int getMaxCooccsFeatures()
  {
    return this.maxCooccsFeatures;
  }
  public void setMaxCooccsFeatures(int newVal)
  {
    this.maxCooccsFeatures = newVal;
  }

  public int getMaxIndexPosition()
  {
    return this.maxIndexNr;
  }
  public void setMaxIndexPosition(int nr)
  {
    this.maxIndexNr=nr;
  }

  public int getMinIndexPosition()
  {
    return this.minIndexNr;
  }
  public void setMinIndexPosition(int nr)
  {
    this.minIndexNr=nr;
  }


  public String getOutputMainString()
  {
    return this.outputMainString;
  }
  public void setOutputMainString(String newString)
  {
    this.outputMainString = newString;
  }

  public int getResultsFactor()
  {
    return this.resultsFactor;
  }

  public int getMinWordNr()
  {
    return this.minWordNr;
  }
  public void setMinWordNr(int newVal)
  {
    this.minWordNr = newVal;
  }

  public int getMaxResultsSizePerWord()
  {
    return this.maxResultSize;
  }
  public void setMaxResultsSizePerWord(int newSize)
  {
    this.maxResultSize = newSize;
  }

  public int getMinAnzahl()
  {
    return this.minAnzahl;
  }
  public void setMinAnzahl(int anzahl)
  {
    this.minAnzahl = anzahl;
  }

  public int[] getMinCols()
  {
    return minCols;
  }
  public void setMinCols(int[] mc)
  {
    minCols = mc;
    for ( int i = 0 ; i < minCols.length ; i++ )
    {
      minCols[i] = minCols[i];//*this.inputDataFactor;
    }
  }

  public int[] getMaxCols()
  {
    return maxCols;
  }
  public void setMaxCols(int[] mc)
  {
    maxCols = mc;
  }
}
