/**
 * This file was adapted for the UnsuPos system. Some annotations <code>@SuppressWarnings("unchecked")</code>
 * were added. Furthermore the <code>printDBResults()</code> method of the <code>RankedResultList</code>
 * class was modified to write only necessary information.
 * 
 * @author Andreas Klaus 
 */

package de.uni_leipzig.asv.unsupos.similarity;

//import de.uni_leipzig.asv.wordserver.*;
import de.uni_leipzig.asv.coocc.*;

import java.util.*;
import java.io.*;

/**
 * @todo Rechnet zwei mal das gleiche aus!?
 *
 *
 * @author Stefan Bordag
 * @date   15.11.2003
 */
public class Similarity
{
  protected static final String A_factor = "factor";
  protected static final String A_inputFactor = "inputfactor";
  protected static final String A_CueToRam = "cuetoram";
  protected static final String A_FeatureToRam = "featuretoram";
  protected static final String A_takeColls = "takecolls";
  protected static final String A_cooccsCue = "cooccscue";
  protected static final String A_cooccsFeature = "cooccsfeature";
  protected static final String A_minSig = "minsig";
  protected static final String A_maxResults = "maxresults";
  protected static final String A_minAnzahl = "minanzahl";
  protected static final String A_outstring = "outpath";

  protected BitField doneWords = null;

  /**
   * This is the source of information on which candidates might be good.
   * from this, the first and second order neighbours will be fetched. (As
   * specified in the options, such that a direct candidate source might be
   * used)
   */
  protected BinFileMultCol cueSource = null;

  /**
   * From here the features will be taken, merged in the Merger class and the
   * used to compare items which came from the cueSource
   */
  protected BinFileMultCol[] featureSources = null;
  
  private SimOptions simopt = null;
  public int taskProgress = 0;

  public Similarity(SimOptions simopt, BinFileMultCol dataSource)
  {
	this.simopt    = simopt;
    this.cueSource = dataSource;
    this.featureSources = new BinFileMultCol[1];
    this.featureSources[0] = dataSource;
    init();
  }

  public Similarity(SimOptions simopt, BinFileMultCol cueSource, BinFileMultCol[] featureSources)
  {
	this.simopt    = simopt;
    this.cueSource = cueSource;
    this.featureSources = featureSources;
    init();
  }

  protected void init()
  {
    simopt.setMaxIndexPosition(this.cueSource.getMaxIndexPosition().intValue());
    simopt.setMinIndexPosition(this.cueSource.getFirstNonNullPosition().intValue());

    this.doneWords = new BitField(simopt.getMaxIndexPosition()+1);

    int[] minCols = new int[3];
    minCols[0] = 0;
    minCols[1] = simopt.getMinWordNr();
    minCols[2] = simopt.getMinSignifikance();
    simopt.setMinCols(minCols);

    int[] maxCols = new int[3];
    maxCols[0] = Integer.MAX_VALUE;
    maxCols[1] = Integer.MAX_VALUE;
    maxCols[2] = Integer.MAX_VALUE;
    simopt.setMaxCols(maxCols);
  }

  public void runAlgorithm()
  {
    int minNr = simopt.getMinIndexPosition();
    int maxNr = simopt.getMaxIndexPosition();

    SimAlgorithm simAlg = new SimAlgorithm(simopt, this.doneWords, this.cueSource, this.featureSources);

    System.out.println("Beginning computations.\n" + simopt);

    int i=0;
    double diff = Math.abs((double) (maxNr - minNr)) / 2.0;
    int cnt = 0;
    taskProgress = 0;
    for ( i = maxNr ; i >= minNr ; i-- ) {
      if (cnt++ >= (int) diff) {
    	  taskProgress = Math.min(taskProgress + 14, 100);
    	  diff = diff / 2.0;
    	  cnt = 0; }
      
      //if ( i % 100 == 0 ) {
        //System.out.print(".");
        
        /* if ( i % 1000 == 0 ) {
               System.out.println("i=" + i); }
        */
      //}
      RankedResultList l = simAlg.computeSimilarity( new Integer( i ) );
      
      l.printResults();
      this.doneWords.setPosition( i, true );
    }
    taskProgress = 100;
    if ( simopt.getShowProgress() ) { System.out.println("Done with words! strange... "+i); }
  }


  public static void main(String args) {
    main(args.split(" ")); }

  public static boolean existsParam(List args, String arg)
  {
    if ( args.contains(arg) && args.size() > args.indexOf(arg) )
    {
      return true;
    }
    else if ( args.contains(arg) && args.size() == (args.indexOf(arg)-1) )
    {
      System.out.println("Argument "+arg+" needs a value specified.");
      System.exit(0);
    }
    return false;
  }

  public static List removeParam(List arguments,String arg)
  {
    System.out.println("Setting "+arguments.get(arguments.indexOf(arg))+" to "+arguments.get(arguments.indexOf(arg)+1)+".");
    arguments.remove(arguments.indexOf(arg)+1);
    arguments.remove(arguments.indexOf(arg));
    return arguments;
  }

  public static boolean fileExists(String file)
  {
    if ( new File(file).exists() && new File(file).length() > 0 )
    {
      return true;
    }
    return false;
  }

  /**
   *
   * @param args
   */
  @SuppressWarnings("unchecked")
  public static void main(String[] args)
  {
    SimOptions simopt = new SimOptions();

    List arguments = new ArrayList();
    for ( int i = 0 ; i < args.length ; i++ )
    {
      arguments.add(args[i]);
    }
    if ( existsParam(arguments,A_cooccsCue) )
    {
      simopt.setMaxCooccsCandidate(new Integer((String)arguments.get(arguments.indexOf(A_cooccsCue)+1)).intValue());
      arguments = removeParam(arguments,A_cooccsCue);
    }
    if ( existsParam(arguments,A_cooccsFeature) )
    {
      simopt.setMaxCooccsFeatures(new Integer((String)arguments.get(arguments.indexOf(A_cooccsFeature)+1)).intValue());
      arguments = removeParam(arguments,A_cooccsFeature);
    }
    if ( existsParam(arguments,A_CueToRam) )
    {
    	simopt.setLoadCueToRam(new Boolean((String)arguments.get(arguments.indexOf(A_CueToRam)+1)).booleanValue());
      arguments = removeParam(arguments,A_CueToRam);
    }
    if ( existsParam(arguments,A_FeatureToRam) )
    {
    	simopt.setLoadFeaturesToRam(new Boolean((String)arguments.get(arguments.indexOf(A_FeatureToRam)+1)).booleanValue());
      arguments = removeParam(arguments,A_FeatureToRam);
    }
    if ( existsParam(arguments,A_factor) )
    {
    	simopt.setFactor(new Integer((String)arguments.get(arguments.indexOf(A_factor)+1)).intValue());
      arguments = removeParam(arguments,A_factor);
    }
    if ( existsParam(arguments,A_inputFactor) )
    {
    	simopt.setInputDataFactor(new Integer((String)arguments.get(arguments.indexOf(A_inputFactor)+1)).intValue());
      arguments = removeParam(arguments,A_inputFactor);
    }
    if ( existsParam(arguments,A_maxResults) )
    {
    	simopt.setMaxResultsSizePerWord(new Integer((String)arguments.get(arguments.indexOf(A_maxResults)+1)).intValue());
      arguments = removeParam(arguments,A_maxResults);
    }
    if ( existsParam(arguments,A_minAnzahl) )
    {
    	simopt.setMinAnzahl(new Integer((String)arguments.get(arguments.indexOf(A_minAnzahl)+1)).intValue());
      arguments = removeParam(arguments,A_minAnzahl);
    }
    if ( existsParam(arguments,A_minSig) )
    {
    	simopt.setMinSignifikance(new Integer((String)arguments.get(arguments.indexOf(A_minSig)+1)).intValue());
      arguments = removeParam(arguments,A_minSig);
    }
    if ( existsParam(arguments,A_outstring) )
    {
    	simopt.setOutputMainString((String)arguments.get(arguments.indexOf(A_outstring)+1));
      arguments = removeParam(arguments,A_outstring);
    }
    if ( existsParam(arguments,A_takeColls) )
    {
    	simopt.setTakeFurther(new Boolean((String)arguments.get(arguments.indexOf(A_takeColls)+1)).booleanValue());
      arguments = removeParam(arguments,A_takeColls);
    }

    String cueFile = "test/kollok_sig.txt";
    Similarity sim = null;
    // remaining strings are interpreted as follows:
    // first is cue, all remaining are features, if no features, then cue=feature
    if ( arguments.size() == 0 )
    {
      System.out.println("Please specify a file to be processed");
    }
    else if ( arguments.size() == 1 )
    {
      cueFile = (String)arguments.get(0);
      if ( !fileExists(cueFile) )
      {
        System.out.println("Specified input file "+cueFile+" does not exist or is empty.");
        System.exit(0);
      }
      System.out.println("Taking "+cueFile+" as cue- and datasource. in ram ("+simopt.getLoadFeaturesToRam()+")");
      BinFileMultCol data = BinFileConstructor.getBinFileMultCol(cueFile,3,simopt.getLoadFeaturesToRam());
      sim = new Similarity(simopt, data);
    }
    else if ( arguments.size() > 1 )
    {
      for ( Iterator it = arguments.iterator() ; it.hasNext() ; )
      {
        String curFile = (String)it.next();
        if ( ! fileExists(curFile) )
        {
          System.out.println("Specified input file "+curFile+" does not exist or is empty.");
          System.exit(0);
        }
      }
      cueFile = (String)arguments.get(0);
      System.out.println("Taking "+cueFile+" as cue data.");
      BinFileMultCol cueData = BinFileConstructor.getBinFileMultCol(cueFile,3,simopt.getLoadCueToRam());
      arguments.remove(0);
      BinFileMultCol[] featureFiles = new BinFileMultCol[arguments.size()];
      int i = 0;
      for ( Iterator it = arguments.iterator() ; it.hasNext() ; i++)
      {
        String curFile = (String) it.next();
        System.out.println("Taking "+curFile+" as features data.");
        featureFiles[i] = BinFileConstructor.getBinFileMultCol(curFile,3,simopt.getLoadFeaturesToRam());
      }
      sim = new Similarity(simopt, cueData, featureFiles);
    }
    sim.runAlgorithm();
  }
}
