package newsProcessor;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.commons.math3.analysis.function.Add;

import stockPrices.TimeSeries;
import trends.CheckRegressionTrends;
import trends.ClosestTime;
import trends.RegressionObject;

public class MatchNewsWithTrendRegression implements Callable<RegressionObject>{
	
	int index, offset, daydiff, lengthoftrendinminutes;
	String time, newsDate, symbol, newsname;
	RegressionObject OutputRo;
	public RegressionObject getOutputRo() {
		return OutputRo;
	}


	public void setOutputRo(RegressionObject outputRo) {
		OutputRo = outputRo;
	}


	public String getNewsname() {
		return newsname;
	}


	public void setNewsname(String newsname) {
		this.newsname = newsname;
	}


	public String getSymbol() {
		return symbol;
	}


	public void setSymbol(String symbol) {
		this.symbol = symbol;
	}

	boolean afterclosing;
	TimeSeries ts;
	
	public int getIndex() {
		return index;
	}


	public void setIndex(int index) {
		this.index = index;
	}


	public int getOffset() {
		return offset;
	}


	public void setOffset(int offset) {
		this.offset = offset;
	}


	public int getDaydiff() {
		return daydiff;
	}


	public void setDaydiff(int daydiff) {
		this.daydiff = daydiff;
	}


	public int getLengthoftrendinminutes() {
		return lengthoftrendinminutes;
	}


	public void setLengthoftrendinminutes(int lengthoftrendinminutes) {
		this.lengthoftrendinminutes = lengthoftrendinminutes;
	}


	public String getTime() {
		return time;
	}


	public void setTime(String time) {
		this.time = time;
	}


	public String getNewsDate() {
		return newsDate;
	}


	public void setNewsDate(String newsDate) {
		this.newsDate = newsDate;
	}


	public boolean isAfterclosing() {
		return afterclosing;
	}


	public void setAfterclosing(boolean afterclosing) {
		this.afterclosing = afterclosing;
	}


	public TimeSeries getTs() {
		return ts;
	}


	public void setTs(TimeSeries ts) {
		this.ts = ts;
	}


	public MatchNewsWithTrendRegression(int index,int offset,int daydiff,int length, String time, String newsDate,boolean afterclosing, TimeSeries ts, String filename, String newsName){
		
		setIndex(index);
		setOffset(offset);
		setDaydiff(daydiff);
		setLengthoftrendinminutes(length);
		setTime(time);
		setNewsDate(newsDate);
		setAfterclosing(afterclosing);
		setTs(ts);
		setSymbol(filename);
		setNewsname(newsName);
	}
	
	
	public static int getTrend(File file) throws FileNotFoundException, IOException, ClassNotFoundException, InterruptedException, ExecutionException{
		int maxlength = 240;
		List<Future<RegressionObject>> list = Collections.synchronizedList(new ArrayList<Future<RegressionObject>>());
		File[] dirList = file.listFiles();
		if(dirList.length==0){
			return -2;
		}
		TimeSeries ts = null;
		
		int totCount =0;
		int eCount = 0;
		ExecutorService exService = new ThreadPoolExecutor(3, 3, 10, TimeUnit.SECONDS, new ArrayBlockingQueue<Runnable>(6, true), new ThreadPoolExecutor.CallerRunsPolicy());
		for(int i = 0; i<dirList.length;i++){
			ObjectInputStream ois = new ObjectInputStream(new BufferedInputStream(new FileInputStream(new File("c:/master/timeseries/"+dirList[i].getName()))));
			ts = (TimeSeries) ois.readObject();
			File[] subDirList = dirList[i].listFiles();
			System.out.println(dirList[i].getName());
			for(int j = 0; j < subDirList.length;j++){
				
				
				ObjectInputStream oisNews = new ObjectInputStream(new BufferedInputStream(new FileInputStream(subDirList[j])));
				News news = (News) oisNews.readObject();
				ClosestTime ct = new ClosestTime(news.getDate(), ts);
				if(ct.isFalse){
					eCount++;
					continue;
				}
				if(ct.getTransactionID()==0){
					eCount++;
					continue;
				}
				for(int lengthOfTrendInMinutes = 1; lengthOfTrendInMinutes<=maxlength; lengthOfTrendInMinutes++){
					try{
						
						Callable<RegressionObject> t = new MatchNewsWithTrendRegression(ct.getTransactionID(), ct.getOffset(), ct.getDayDiff(), lengthOfTrendInMinutes, ts.getTime().get(ct.getTransactionID()), "", ct.isAfterClosing, ts, dirList[i].getName(), subDirList[j].getName());
						Future<RegressionObject> submit = exService.submit(t);
						list.add(submit);
					} catch (IndexOutOfBoundsException e) {
						// TODO: handle exception
						e.printStackTrace();
						System.out.println(ct.getTransactionID());
					}
					
				}
				oisNews.close();
			}
			ois.close();
		}
		
		
//		exService.awaitTermination(30, TimeUnit.SECONDS);
		exService.shutdown();
		System.out.println("ERROR COUNT: "+eCount);
		ArrayList<ArrayList<RegressionObject>> rOList = new ArrayList<>();
		for(int count = 1; count<=maxlength;count++){
			rOList.add(new ArrayList<RegressionObject>());
		}
		
		for(int i = 0; i < list.size();i++){
//			System.out.println(list.size());
			RegressionObject tempRO = list.get(i).get();
			if(tempRO==null){
				continue;
			}			
			int length = tempRO.getLength();
//			System.out.println(length);
			ArrayList<RegressionObject> tempList = rOList.get(length-1);
			if(tempList==null){
				tempList = new ArrayList<RegressionObject>();
			}
			tempList.add(tempRO);
			rOList.set(length-1, tempList);
		}
		
		HashMap<String,RegressionObject> map = new HashMap<>();
		
		for(int i = 0; i < rOList.size();i++){
			for(int j = 0; j < rOList.get(i).size();j++){
				totCount++;
				String key = "" + rOList.get(i).get(j).getNameOfNews()+"min"+i;
				RegressionObject value = rOList.get(i).get(j);
				map.put(key, value);
				
			}
		}
		
		System.out.println("Tot count: " + totCount);
		System.out.println("list size " + list.size());
		System.out.println(map.size());
		
		File outputFile = new File("c:/master/regressionResults/regressionResults");
		if(!outputFile.getParentFile().exists()){
			outputFile.getParentFile().mkdirs();
		}
		
		ObjectOutputStream oos = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(outputFile)));
		oos.writeObject(map);
		oos.close();
		
		return 1;
		
	}
	
//	public void run() throws IOException{
//		File outputFile = new File("c:/master/regressionResults/"+getLengthoftrendinminutes()+"/"+getSymbol()+"/"+getNewsname());
//		File dirMaker = new File("c:/master/regressionResults/"+getLengthoftrendinminutes()+"/");
//		File dirMaker2 = new File("c:/master/regressionResults/"+getLengthoftrendinminutes()+"/"+getSymbol()+"/");
//		
//		if(outputFile.exists()){
//			RegressionObject ro = null;
//			return;
//		}
//		if(!dirMaker.exists()){
//			dirMaker.mkdir();
//		}
//		if(!dirMaker2.exists()){
//			dirMaker2.mkdir();
//		}
		
		
		
		
//	}
	@Override
	public RegressionObject call(){
		RegressionObject ro = CheckRegressionTrends.getTrendRegression(getIndex(), getTime(), getOffset(), getDaydiff(), isAfterclosing(), getNewsDate(), getLengthoftrendinminutes(), getTs(),getNewsname(),getSymbol());
		return ro;
	}

}
